# Generated by Django 2.2.11 on 2021-05-14 15:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('media', '0012_auto_20210418_2350'),
    ]

    operations = [
        migrations.AddField(
            model_name='video',
            name='duration',
            field=models.DurationField(blank=True, null=True),
        ),
    ]
