# Generated by Django 2.2.9 on 2020-10-19 17:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pages', '0013_auto_20201013_1619'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='page',
            unique_together=set(),
        ),
        migrations.AddConstraint(
            model_name='page',
            constraint=models.UniqueConstraint(condition=models.Q(('version_for_id__isnull', True), ('country_group__isnull', True)), fields=('parent', 'slug'), name='unique_page_urls'),
        ),
        migrations.AddConstraint(
            model_name='page',
            constraint=models.UniqueConstraint(condition=models.Q(version_for_id__isnull=True), fields=('country_group', 'owner'), name='unique_language_versions'),
        ),
    ]
