# Generated by Django 2.2.9 on 2020-11-11 15:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pages', '0017_page_version_name'),
    ]

    operations = [
        migrations.AddField(
            model_name='page',
            name='version_publication_date',
            field=models.DateTimeField(blank=True, help_text='The date this version will be published', null=True, verbose_name='publish on'),
        ),
    ]
