# Generated by Django 2.2.9 on 2020-11-02 12:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pages', '0014_auto_20201019_1828'),
    ]

    operations = [
        migrations.AlterField(
            model_name='page',
            name='left',
            field=models.IntegerField(db_index=True, editable=False, null=True),
        ),
        migrations.AlterField(
            model_name='page',
            name='right',
            field=models.IntegerField(db_index=True, editable=False, null=True),
        ),
    ]
